import random
import httpx
from httpx import ConnectError, ConnectTimeout, ProxyError, ReadTimeout, ReadError, UnsupportedProtocol
from scan.response import Response
from scan.common import logger


class Downloader(object):
    def __init__(self):
        self.client = None

    @staticmethod
    async def gen_headers():
        kit = f'{random.randint(490,540)}.{random.randint(10,90)}'
        return {
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;'
                      'q=0.8,application/signed-exchange;v=b3;q=0.9',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en;q=0.9',
            'user-agent': f'Mozilla/5.0 (Macintosh; Intel Mac OS X {random.randint(6,15)}_{random.randint(2,20)}'
                          f'_{random.randint(1,9)}) AppleWebKit/{kit} (KHTML, like Gecko) '
                          f'Chrome/{random.randint(95, 98)}.0.{random.randint(1000,5000)}.{random.randint(100, 200)}'
                          f' Safari/{kit}'
        }

    async def close(self):
        try:
            await self.client.aclose()
        except:
            pass

    @staticmethod
    async def log_request(request):
        pass

    @staticmethod
    async def log_response(response):
        """
        日志钩子
        """
        request = response.request
        if response.status_code not in [200, 301, 302]:
            logger.error(f'{response.status_code}  {request.url}')
        else:
            logger.info(f'{response.status_code}  {request.url}')

    async def request(self, url, params=None, headers=None, cookies=None, auth=None, proxies=None, allow_redirects=True,
                      verify=True, http2=False,  content=None, data=None, files=None, json=None, stream=False,
                      timeout=30, cycle=3):
        if data or json or content:
            method = 'POST'
        else:
            method = 'GET'
        if not headers:
            headers = await self.gen_headers()
        response = Response()
        response.request_url = url
        for _ in range(cycle):
            try:
                if stream:
                    client = httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]},
                                               verify=verify, http2=http2, follow_redirects=allow_redirects)
                    request = client.build_request(method=method, url=url, headers=headers, cookies=cookies,
                                                   content=content, data=data, files=files, json=json, timeout=timeout)
                    resp = await client.send(request, stream=True)
                    response.response = resp
                    response.ok = True
                    response.client = client  # need aclose
                else:
                    async with httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]},
                                                 verify=verify, http2=http2) as client:
                        resp = await client.request(
                            method=method, url=url, content=content, data=data, files=files, json=json, params=params,
                            headers=headers, cookies=cookies, auth=auth, follow_redirects=allow_redirects,
                            timeout=timeout
                        )
                        response.response = resp
                        response.ok = True
                return response
            except ConnectError as e:
                response.message = 'ConnectError'
                logger.error(f'Failed to request {url}  ConnectError:{e}')
            except ConnectTimeout as e:
                response.message = 'ConnectTimeout'
                logger.error(f'Failed to request {url}  ConnectTimeout:{e}')
            except ProxyError as e:
                response.message = 'ProxyError'
                logger.error(f'Failed to request {url}  ProxyError:{e}')
            except ReadTimeout as e:
                response.message = 'ReadTimeout'
                logger.error(f'Failed to request {url}  ReadTimeout:{e}')
            except ReadError as e:
                response.message = 'ReadError'
                logger.error(f'Failed to request {url}  ReadError:{e}')
            except UnsupportedProtocol as e:
                response.message = 'UnsupportedProtocol'
                logger.error(f'Failed to request {url}  UnsupportedProtocol:{e}')
            except Exception as e:
                response.message = e
                logger.error(f'Failed to request {url}  {e}')
        return response







import json
import aio_pika
from scan.common import logger
from json import JSONDecodeError


class RabbitMQ:
    def __init__(self, **kwargs):
        self.host = kwargs.get('host') or 'localhost'
        self.port = kwargs.get('port') or 5672
        self.user = kwargs.get('user') or 'root'
        self.password = kwargs.get('password') or 'root'
        self.virtualhost = kwargs.get('virtualhost') or '/'
        self.loop = None
        self.max_pool_size = 1
        self.connection = None

    async def init(self, max_pool_size=None):
        if max_pool_size and isinstance(max_pool_size, int):
            self.max_pool_size = max_pool_size
        await self.get_connection()

    async def get_connection(self):
        if self.connection:
            try:
                await self.connection.close()
            except:
                pass
        self.connection = await aio_pika.connect_robust(host=self.host, port=int(self.port), login=self.user,
                                                        password=self.password, virtualhost=self.virtualhost)

    async def get_channel(self):
        for _ in range(3):
            try:
                if not self.connection:
                    await self.get_connection()
                channel = await self.connection.channel()
                return channel
            except Exception as e:
                logger.error(f'get channel error:{e}')
                self.connection = None

    async def consume(self, call_back,  queue_name, no_ack=False, durable=True, auto_delete=False, arguments=None, qos=1):
        """
        :param call_back: 回调函数
        :param qos:
        :param auto_delete:
        :param durable:
        :param no_ack:
        :param arguments: mq绑定参数
        :param queue_name: 队列名
        :return:
        """
        try:
            channel = await self.get_channel()
            async with channel:
                await channel.set_qos(qos)
                queue = await channel.declare_queue(queue_name, durable=durable, arguments=arguments,
                                                    auto_delete=auto_delete)
                async with queue.iterator(no_ack=no_ack) as queue_iter:
                    async for message in queue_iter:
                        body = message.body
                        try:
                            task_info = json.loads(body.decode())
                        except JSONDecodeError:
                            logger.error(f'Description Failed to format task data:{body}')
                            await message.ack()
                            continue
                        try:
                            pres = await call_back(task_info)
                        except Exception as e:
                            logger.error(f'consume error:{e}')
                        if pres:
                            await message.ack()
                        else:
                            logger.error(f'task fail, resend data:{task_info}')
                            await self.publish(data=task_info, routing_key=queue_name)
                            await message.ack()
        except Exception as e:
            logger.error(f'consume process error: {e}')

    async def publish(self, data, routing_key, priority=None):
        """
        :param priority: 消息优先级
        :param data: 要发送的数据
        :param routing_key: 队列名
        :return:
        """
        try:
            channel = await self.get_channel()
            async with channel:
                data = json.dumps(data)
                for _ in range(3):
                    try:
                        await channel.default_exchange.publish(aio_pika.Message(body=data.encode(), priority=priority),
                                                               routing_key=routing_key)
                        return True
                    except Exception as e:
                        logger.error(f'publish error: {e}')
        except Exception as e:
            logger.error(f'publish process error: {e}')

    async def purge(self, queue_name, arguments=None):
        """
        :param arguments: 绑定队列参数
        :param queue_name: 要清空的队列名
        :return:
        """
        try:
            channel = await self.get_channel()
            async with channel:
                try:
                    queue = await channel.declare_queue(queue_name, durable=True, arguments=arguments,
                                                        auto_delete=False)
                    await queue.purge()
                    return True
                except Exception as e:
                    logger.error(f'purge error: {e}')
        except Exception as e:
            logger.error(f'purge process error: {e}')

    async def message_count(self, queue_name, arguments=None):
        try:
            channel = await self.get_channel()
            async with channel:
                try:
                    queue = await channel.declare_queue(queue_name, durable=True, arguments=arguments,
                                                        auto_delete=False)
                    count = queue.declaration_result.message_count
                    return count
                except Exception as e:
                    logger.error(f'message_count error: {e}')
        except Exception as e:
            logger.error(f'message_count process error: {e}')


__all__ = RabbitMQ











import json
import chardet
from bs4 import BeautifulSoup
from scan.common import logger


class Response:
    def __init__(self):
        self.__req_resp = None
        self.status_code = None
        self.message = ''
        self.ok = False
        self.client = None
        self.request_url = None

    async def aclose(self):
        if self.__req_resp:
            try:
                await self.__req_resp.aclose()
            except:
                pass
        if self.client:
            try:
                await self.client.aclose()
            except:
                pass

    @property
    def response(self):
        return self.__req_resp

    @response.setter
    def response(self, req_resp):
        self.__req_resp = req_resp
        self.status_code = req_resp.status_code

    def json(self):
        try:
            return json.loads(self.__req_resp.content)
        except Exception as e:
            logger.error(f'格式化json异常:{e}, 链接:{self.request_url}')

    def soup(self):
        try:
            soup = BeautifulSoup(self.__req_resp.content, 'lxml')
            return soup
        except Exception as e:
            logger.error(f'格式化soup异常:{e}, 链接:{self.request_url}')

    def text(self):
        try:
            return self.__req_resp.content.decode()
        except UnicodeDecodeError:
            try:
                encoding = chardet.detect(self.__req_resp.content).get('encoding')
                return self.__req_resp.content.decode(encoding)
            except Exception as e:
                logger.error(f'格式化text异常:{e}')
        except Exception as e:
            logger.error(f'格式化text异常:{e}, 链接:{self.request_url}')

    def content(self):
        try:
            content = self.__req_resp.content
            return content
        except Exception as e:
            logger.error(f'获取content异常:{e}, 链接:{self.request_url}')






import json
import time
from scan import crawl
from hashlib import md5
from scan.common import logger


class Monitor:
    def __init__(self):
        self.__logs = {}

    def _need_send(self, send_data):
        data_md5 = md5(json.dumps(send_data).encode()).hexdigest()
        now = int(time.time())
        for k, v in self.__logs.items():
            if v + 60 * 30 < now:
                self.__logs.pop(k)
        if self.__logs.get(data_md5):
            return False
        else:
            self.__logs.update({data_md5: int(time.time())})
            return True

    async def send_warn_fs(self, web_hook_url, product_name, message, at='', uid=''):
        post_data = {
            "msg_type": "post",
            "content": {
                "post": {
                    "zh_cn": {
                        "title": "爬虫项目异常推送",
                        "content": [
                            [{"tag": "text", "text": f"项目:【{product_name}】"}],
                            [{"tag": "text", "text": f"描述: {message}"}],
                            [{"tag": "text", "text": f"项目负责人:@{at}"}]
                        ]
                    }
                }
            }
        }
        if not self._need_send(post_data):
            return
        try:
            resp = await crawl.fetch(web_hook_url, json=post_data)
            data = resp.json()
            logger.info(f'告警返回:{data}')
        except Exception as e:
            logger.error(f'发送告警数据异常:{e}')

    async def async_check_mysql_update_time(self, conn, db, table):
        pass

    def check_mysql_update_time(self, conn, db, table):
        pass


monitor = Monitor()
__all__ = monitor



import asyncpg
from scan.common import logger


class Postgresql:
    def __init__(self, host, port, user, database, password, min_size=10, max_size=10, **kwargs):
        self._pool = None
        self.host = host
        self.port = port
        self.user = user
        self.database = database
        self.password = password
        self.min_size = min_size
        self.max_size = max_size

    async def _create_pool(self):
        try:
            dsn = f'postgres://{self.user}:{self.password}@{self.host}:{self.port}/{self.database}'
            self._pool = await asyncpg.create_pool(dsn=dsn, min_size=self.min_size, max_size=self.max_size,
                                                   command_timeout=60)
        except Exception as e:
            logger.error(f'create pool error: {e}')

    async def get_pool(self):
        if not self._pool:
            await self._create_pool()
        return self._pool


__all__ = Postgresql





from motor.motor_asyncio import AsyncIOMotorClient, AsyncIOMotorDatabase, AsyncIOMotorCollection


class MongoDB:
    def __init__(self, **kwargs):
        self.host = kwargs.get('host') or 'localhost'
        self.port = kwargs.get('port') or 6379
        self.user = kwargs.get('user') or 'admin'
        self.password = kwargs.get('password') or 'admin'
        self.auth_db = kwargs.get('auth_db') or 'admin'
        self.client = None

    def create_client(self) -> AsyncIOMotorClient:
        if not self.user:
            c = AsyncIOMotorClient(self.host, self.port)
        else:
            url = f'mongodb://{self.user}:{self.password}@{self.host}:{self.port}/?authSource={self.auth_db}'
            c = AsyncIOMotorClient(url)
        return c

    def db(self, db_name) -> AsyncIOMotorDatabase:
        if not self.client:
            self.client = self.create_client()
        db = self.client[db_name]
        return db

    def collection(self, db_name, coll_name) -> AsyncIOMotorCollection:
        db = self.db(db_name)
        coll = db[coll_name]
        return coll


__all__ = MongoDB








from scan.database.rabbitmq import RabbitMQ
from scan.database.redis import Redis
from scan.database.mongodb import MongoDB
from scan.database.oss import OSS
from scan.database.pg import Postgresql
from scan.database.kafka import Kafka


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hscan",
    version="2.4.0",
    author="jyanghe",
    author_email="jyanghe1023@gmail.com",
    description="A python framework",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jyangHe/hscan",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8.0',
    install_requires=[
        'httpx[http2]==0.22.0',
        'aiofiles==0.7.0',
        'aio-pika==6.8.0',
        'beautifulsoup4==4.9.3',
        'aioredis==2.0.0',
        'motor==2.5.1',
        'Brotli==1.0.9',
        'pymongo==3.12.0',
        'chardet==4.0.0',
        'asyncpg==0.26.0',
        'aiokafka==0.8.0'
    ]
)



import json
import zlib
from aiokafka import AIOKafkaProducer, AIOKafkaConsumer, TopicPartition
from scan.common import logger


class Kafka:

    def __init__(self, bootstrap_servers, task_queue, security_protocol='PLAINTEXT',
                 group_id='hscan', ssl_context=None, user=None, password=None, api_version='auto'):
        self.task_queue = task_queue
        self.bootstrap_servers = bootstrap_servers
        self.user = user
        self.password = password
        self.ssl_context = ssl_context
        self.security_protocol = security_protocol
        self.api_version = api_version
        self.group_id = group_id

    async def producer(self, gen_func, topic=None, **kwargs):
        """
        :param topic:
        :param gen_func:  generator func
        :return:
        """
        _producer = AIOKafkaProducer(
            bootstrap_servers=self.bootstrap_servers,
            security_protocol=self.security_protocol,
            api_version=self.api_version,
            ssl_context=self.ssl_context,
            sasl_plain_username=self.user,
            sasl_plain_password=self.password,
            **kwargs
        )
        await _producer.start()
        if not topic:
            topic = self.task_queue
        try:
            data = gen_func()
            async for d in data:
                if not isinstance(d, str):
                    d = json.dumps(d)
                await _producer.send_and_wait(topic, d.encode())
                logger.info(f'send data: {d}')
        except Exception as e:
            logger.error(f'producer error:{e}')
        finally:
            await _producer.stop()

    async def send_one(self, message,  topic=None, **kwargs):
        """
        :param topic:
        :param message:
        :return:
        """
        _producer = AIOKafkaProducer(
            bootstrap_servers=self.bootstrap_servers,
            security_protocol=self.security_protocol,
            api_version=self.api_version,
            ssl_context=self.ssl_context,
            sasl_plain_username=self.user,
            sasl_plain_password=self.password,
            **kwargs
        )
        await _producer.start()
        if not topic:
            topic = self.task_queue
        try:
            if isinstance(message, dict):
                message = json.dumps(message)
            if isinstance(message, bytes):
                await _producer.send_and_wait(topic, message)
            else:
                await _producer.send_and_wait(topic, message.encode())
        except Exception as e:
            logger.error(f'producer error:{e}')
        finally:
            await _producer.stop()

    async def consumer(self, call_back, topic=None, group_id=None, auto_offset_reset='earliest', **kwargs):
        """
        :param call_back: async func
        :param topic:
        :param group_id:
        :param auto_offset_reset:
        :return:
        """
        try:
            if not topic:
                topic = self.task_queue
            _consumer = AIOKafkaConsumer(
                topic,
                bootstrap_servers=self.bootstrap_servers,
                group_id=group_id if group_id else self.group_id,
                auto_offset_reset=auto_offset_reset,
                enable_auto_commit=False,
                sasl_plain_username=self.user,
                sasl_plain_password=self.password,
                **kwargs
            )
            await _consumer.start()
            try:
                # while 1:
                    # msg = await _consumer.getone()
                    # if not msg:
                    #     continue
                    # data = json.loads(msg.value)
                    # res = await call_back(data)
                    # if res:
                    #     tp = TopicPartition(msg.topic, msg.partition)
                    #     await _consumer.commit({tp: msg.offset + 1})
                async for msg in _consumer:
                    try:
                        data = json.loads(msg.value)
                    except UnicodeDecodeError:
                        data = json.loads(zlib.decompress(msg.value).decode())
                    tp = TopicPartition(msg.topic, msg.partition)
                    await _consumer.commit({tp: msg.offset + 1})
                    # if data is None:
                    #     tp = TopicPartition(msg.topic, msg.partition)
                    #     await _consumer.commit({tp: msg.offset + 1})
                    #     continue
                    res = await call_back(data)
                    # if res:
                        # tp = TopicPartition(msg.topic, msg.partition)
                        # await _consumer.commit({tp: msg.offset + 1})
            except Exception as e:
                logger.error(f'consumer process error:{e}')
            finally:
                await _consumer.stop()
        except Exception as e:
            logger.error(f'consumer error:{e}')

    async def lag(self, topic, group_id='hscan', auto_offset_reset='earliest', **kwargs):
        try:
            _consumer = AIOKafkaConsumer(
                topic,
                bootstrap_servers=self.bootstrap_servers,
                group_id=group_id,
                auto_offset_reset=auto_offset_reset,
                enable_auto_commit=False,
                sasl_plain_username=self.user,
                sasl_plain_password=self.password,
                **kwargs
            )
            await _consumer.start()
            try:
                partitions = [TopicPartition(topic, p) for p in _consumer.partitions_for_topic(topic)]
                # 总偏移
                toff = await _consumer.end_offsets(partitions)
                toff = [(key.partition, toff[key]) for key in toff.keys()]
                toff.sort()
                # 当前消费偏移
                coff = [(x.partition, await _consumer.committed(x)) for x in partitions]
                coff.sort()
                # 堆积量
                toff_sum = sum([x[1] for x in toff])
                cur_sum = sum([x[1] for x in coff if x[1] is not None])
                left_sum = toff_sum - cur_sum
                return left_sum
            finally:
                await _consumer.stop()
        except Exception as e:
            logger.error(f'get heap error:{e}')




import datetime
import asyncio
from abc import ABCMeta, abstractmethod
from pymongo.errors import DuplicateKeyError
from scan import logger
from scan.config import Config
from scan.util import get_local_ip, get_local_name, date_to_char, encrypt


class Spider(object):
    __metaclass__ = ABCMeta

    def __init__(self, spider_name, cfg_path):
        self.local_ip = None
        self.container_id = None
        self.rabbitmq = None
        self.mongo_db = None
        self.redis = None
        self.redis_conn = None
        self.kafka = None
        self.oss_conn = None
        self.postgresql = None
        self.task_num = None
        self.spider_name = spider_name
        self.spider_id = self.gen_spider_id()
        self.config = Config(cfg_path)
        self.status = 1

    def gen_spider_id(self):
        self.local_ip = get_local_ip()
        self.container_id = get_local_name()[:12]
        spider_id = self.spider_name + '-' + self.local_ip + '-' + self.container_id
        with open('spider_id', 'w') as f:
            f.write(spider_id)
            f.close()
        return spider_id

    @abstractmethod
    async def process(self, task_info):
        """
        爬虫业务逻辑入口
        :param task_info: 队列任务信息
        :return:
        """

    async def site(self):
        """
        1. 可以对配置文件进行修改
        eg：
            self.config.config.set('client', 'task_num', 1)
        :return:
        2. 可以设置新的成员变量
        eg:
            self.mq2 = ''
        """

    @abstractmethod
    async def init(self):
        """
        初始化数据处理连接
        eg:
            mq_config = self.config.rabbitmq()
            self.rabbitmq = RabbitMQ(host=mq_config.get('host'), port=mq_config.get('port'), user=mq_config.get('user'),
                           password=mq_config.get('password'))
            await self.rabbitmq.init()
        :return:
        """

    async def on_success(self, msg, mongo_conn=None, collection=None, spider_name=None, desc=None, url=None,
                         headers=None):
        """
        处理成功回调
        """
        if not self.mongo_db and not mongo_conn:
            logger.error('没有mongodb连接信息，不能保存日志')
            return
        if not collection:
            collection = 'spider_success_logs'
        try:
            save_data = {
                "ip": self.local_ip,
                'spider_name': spider_name or self.spider_name,
                'cid': self.container_id,
                'url': url,
                'headers': headers,
                'datetime': str(datetime.datetime.now()),
                'desc': desc,
                'msg': msg
            }
            if mongo_conn:
                await mongo_conn.get_collection(collection).insert_one(save_data)
            else:
                await self.mongo_db.get_collection(collection).insert_one(save_data)
        except Exception as e:
            logger.error(f'处理成功回调异常:{spider_name}-{self.local_ip}-{self.container_id} {e}')

    async def on_error(self, msg, mongo_conn=None, collection=None, spider_name=None, desc=None, url=None, headers=None):
        """
        处理失败日志记录
        :param collection:
        :param mongo_conn:
        :param spider_name:
        :param desc:
        :param msg: traceback信息
        :param url:
        :param headers:
        """
        logger.error(msg)
        if not self.mongo_db and not mongo_conn:
            logger.error('没有mongodb连接信息，不能保存日志')
            return
        if not collection:
            collection = 'spider_error_logs'
        try:
            save_data = {
                "ip": self.local_ip,
                'spider_name': spider_name or self.spider_name,
                'cid': self.container_id,
                'url': url,
                'headers': headers,
                'datetime': str(datetime.datetime.now()),
                'desc': desc,
                'msg': str(msg)
            }
            if mongo_conn:
                await mongo_conn.get_collection(collection).insert_one(save_data)
            else:
                await self.mongo_db.get_collection(collection).insert_one(save_data)
        except Exception as e:
            logger.error(f'处理失败回调异常:{spider_name}-{self.local_ip}-{self.container_id} {e}')

    async def on_count(self, count_type,  mongo_db=None, collection=None, spider_id=None, count=1):
        """
        计数回调
        :param count_type: 1 成功  0 失败
        :param collection:
        :param mongo_db:
        :param spider_id: spider_name-local_ip-container_id
        :param count:
        :return:
        """
        if count_type not in [1, 0]:
            logger.error('状态参数异常，不能计数')
            return
        if not mongo_db and not self.mongo_db:
            logger.error('没有mongodb连接信息，不能计数')
            return
        if not collection:
            collection = 'spider_statistics'
        try:
            # 10分钟一组
            tick = date_to_char('m')[:-1] + '0'
            # 622ea3d44d88e061a3167f33
            # 如 test_spider-192.168.0.182-mac.local-202004261430
            if spider_id:
                count_key = spider_id + '-' + tick
            else:
                count_key = self.spider_id + '-' + tick
            if not mongo_db:
                mongo_db = self.mongo_db
            _id = encrypt(count_key)
            if count_type == 1:
                count_type_key = 'suc_count'
            else:
                count_type_key = 'fail_count'
            try:
                db_data = await mongo_db.get_collection(collection).find_one({'_id': _id})
                if db_data:
                    await mongo_db.get_collection(collection).update_one(
                        {'_id': _id}, {'$inc': {count_type_key: count}})
                else:
                    await mongo_db.get_collection(collection).insert_one(
                        {'_id': _id, 'sid': spider_id, 'date': tick, count_type_key: count})
            except DuplicateKeyError:
                await mongo_db.get_collection(collection).update_one(
                    {'_id': _id}, {'$inc': {'count': count}})
        except Exception as e:
            logger.error(f'处理成功计数回调异常:{spider_id} {e}')

    async def rabbitmq_task(self):
        """
        分布式队列任务，在任务分发使用rabbitmq时
        """
        task_queue = self.config.rabbitmq().get('task_queue')
        if not self.rabbitmq:
            logger.error('The task queue connection is not initialized')
            return
        arguments = None
        # 优先级队列
        if self.config.rabbitmq().get('priority'):
            arguments = {'x-max-priority': int(self.config.rabbitmq().get('priority'))}
        while self.status:
            try:
                await self.rabbitmq.consume(self.process, task_queue, arguments=arguments)
            except Exception as e:
                logger.error(e)

    async def redis_task(self):
        """
        分布式队列任务，在任务分发使用redis时
        """
        task_queue = self.config.redis().get('task_queue')
        if not self.redis:
            logger.error('Need database.redis.Redis class object')
            return
        while self.status:
            try:
                message = await self.redis.consume(task_queue)
                if not message:
                    await asyncio.sleep(10)
                    continue
                res = await self.process(message)
                if not res:
                    await self.redis.resend(message, task_queue)
            except Exception as e:
                logger.error(e)

    async def kafka_task(self):
        if not self.kafka:
            logger.error('Need database.kafka.Kafka class object')
            return
        while self.status:
            try:
                await self.kafka.consumer(call_back=self.process)
            except Exception as e:
                logger.error(e)

    async def simple_task(self):
        """
        单机任务，不使用队列
        """
        while self.status:
            try:
                pres = await self.process(None)
                if pres:
                    pass
            except Exception as e:
                logger.error(e)

    async def run(self, task_num=None):
        """
        爬虫主程序入口
        :param task_num: 并发数
        """
        # 获取爬虫配置
        await self.site()
        # 初始化并发数
        if task_num and isinstance(task_num, int):
            self.task_num = task_num
        else:
            self.task_num = int(self.config.client().get('task_num'))
        # 初始化数据库配置
        await self.init()
        # 根据任务类型，分发任务
        task_type = self.config.client().get('task_type')
        task_list = []
        if task_type == 'simple':
            for _ in range(self.task_num):
                t = asyncio.create_task(self.simple_task())
                task_list.append(t)
        elif task_type == 'redis':
            for _ in range(self.task_num):
                t = asyncio.create_task(self.redis_task())
                task_list.append(t)
        elif task_type == 'kafka':
            for _ in range(self.task_num):
                t = asyncio.create_task(self.kafka_task())
                task_list.append(t)
        else:
            for _ in range(self.task_num):
                t = asyncio.create_task(self.rabbitmq_task())
                task_list.append(t)
        await asyncio.gather(*task_list)

